import requests


class Error(Exception):
    pass


def apply_headers(client):
    headers = []

    headers = {'User-Agent': 'vencode-sdk/py', 'Content-Type': 'application/json',
               'Accept': 'application/json'}
    headers['x-api-key'] = client.options.access.key
    headers['x-user-id'] = client.options.access.id

    return headers


def get(client, path):
    res = requests.get(path, headers=apply_headers(client))
    return res


def post(client, path, **kwargs):
    res = requests.post(client.options.baseUrl + path,
                        headers=apply_headers(client), json=kwargs.get('json'))
    return res



from typing import List
import json


class Credentials:
    def __init__(
        self,
        clientId: str,
        clientSecret: str,
        type: str,
        bucket=None,
        clientEmail=None,
        region=None,
        projectId=None,
        acl=None,
        endpoint=None
    ):
        self.clientId = clientId
        self.clientSecret = clientSecret
        self.type = type
        self.bucket = bucket
        self.clientEmail = clientEmail
        self.region = region
        self.projectId = projectId
        self.acl = acl
        self.endpoint = endpoint

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class Notify:
    def __init__(self, webhookUrl: str):
        self.webhookUrl = webhookUrl

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class Access:
    def __init__(self, key: str, id: str):
        self.key = key
        self.id = id

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class Encode:
    def __init__(self, format, res=None, ac=None, vc=None, asr=None):
        self.res = res
        self.ac = ac
        self.vc = vc
        self.format = format
        self.asr = asr

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class Input:
    def __init__(self, path):
        self.path = path

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class Output:
    def __init__(self, key, encode: Encode, watermark=None):
        self.key = key
        self.encode = encode
        self.watermark = watermark

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class CreateEncodeJobOptions:
    def __init__(self, input: Input, outputs: List[Output]):
        self.input = input
        self.outputs = outputs

    def toJson(self):
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))



import json
import sseclient
import time
from . import api
from . import Access, Notify, Credentials, CreateEncodeJobOptions

BASE_URL = 'https://api.vencode.io/api'


class Options:
    def __init__(self, access: Access, notify: Notify = None, baseUrl=BASE_URL, credentials: Credentials = None, debug: bool = False, maxListenRetry: int = 15):
        self.baseUrl = baseUrl
        self.access = access
        self.notify = notify
        self.credentials = credentials
        self.debug = debug
        self.maxListenRetry = maxListenRetry

    def url(self):
        return self.baseUrl


class Client:
    def __init__(self, options: Options):
        self.options = options
        self.listeners = []

    def with_urllib3(self, url):
        try:
            import urllib3
            http = urllib3.PoolManager()

            res = http.request('GET', url, preload_content=False)

            return res
        except Exception:
            pass

    def with_requests(self, url):
        import requests
        return requests.get(url, stream=True)

    def toJson(self):
        return json.dumps(self, default=lambda o: o.__dict__)

    def encode(self, encode: CreateEncodeJobOptions):
        return api.post(self, "/jobs", json={'credentials': self.options.credentials.toJson(), **encode.toJson()})

    def stop_job(self, id):
        return api.post(self, f"/jobs/{id}/cancel")

    def get_job_metadata(self, id):
        return api.get(self, f"/jobs/{id}")

    def listen_all(self, callback):
        self.listen('ALL', callback=callback)

    def listen(self, id, callback):
        id_entry = "all=true" if id == 'ALL' else f'id={id}'

        while 1:
            response = self.with_urllib3(
                self.options.baseUrl + f'/events?{id_entry}&creds={json.dumps(self.options.access, default=lambda o: o.__dict__)}')
            client = sseclient.SSEClient(response)

            for event in client.events():
                callback(event.data)

            time.sleep(60)
